// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "Evalcorr.h"

bool test1xmm(Attmap& attmap , const double radiusthreshold , const double likhoodthreshold ) 
{
	if ( attmap.find("RAOFFSET") == attmap.end() )
	{
		errstr << "RAOFFSET missing unable to perform 1XMM test" << Sas::error("MissingValue");
	}

	if ( attmap.find("DEOFFSET") == attmap.end() )
	{
		errstr << "DEOFFSET missing unable to perform 1XMM test" << Sas::error("MissingValue");
	}

	errstr << "TEST1XMM" << Sas::message(Msg::App, Msg::Sparse);
	errstr << "RAOFFSET  = " <<  attmap["RAOFFSET"] << Sas::message(Msg::App, Msg::Sparse);
	errstr << "DEOFFSET  = " <<  attmap["DEOFFSET"] << Sas::message(Msg::App, Msg::Sparse);

	double r = sqrt( attmap["RAOFFSET"] * attmap["RAOFFSET"] + attmap["DEOFFSET"] * attmap["DEOFFSET"] );
	errstr << "RADIUS    = " << r << Sas::message(Msg::App, Msg::Sparse);
	errstr << "RADIUSTHRESHOLD = " << radiusthreshold  << Sas::message(Msg::App, Msg::Sparse);

	errstr << "LIK_HOOD  = " <<  attmap["LIK_HOOD"] << Sas::message(Msg::App, Msg::Sparse);
	errstr << "LIKHOODTHRESHOLD = " << likhoodthreshold  << Sas::message(Msg::App, Msg::Sparse);

	double likhood = attmap["LIK_HOOD"];

	bool poscorok;
	poscorok = ( (r <= radiusthreshold) && (likhood > likhoodthreshold) );

	errstr << "RESULT    =  " <<  poscorok << Sas::message(Msg::App, Msg::Sparse);
	return poscorok;
}

bool test2xmm(Attmap& attmap , const double gradient  , const double intercept , double& likhdcmp )
{
	if ( attmap.find("LIK_HOOD") == attmap.end() )
	{
		errstr << "LIK_HOOD missing unable to perform 2XMM test" << Sas::error("MissingValue");
	}

	if ( attmap.find("LIK_NULL") == attmap.end() )
	{
		errstr << "LIK_NULL missing unable to perform 2XMM test" << Sas::error("MissingValue");
	}

	errstr << "TEST2XMM" << Sas::message(Msg::App, Msg::Sparse);
	errstr << "LIK_HOOD   = " << attmap["LIK_HOOD"] << Sas::message(Msg::App, Msg::Sparse);
	errstr << "LIK_NULL   = " << attmap["LIK_NULL"] << Sas::message(Msg::App, Msg::Sparse);
	errstr << "GRADIENT   = " << gradient << Sas::message(Msg::App, Msg::Sparse);
	errstr << "INTERCEPT  = " << intercept << Sas::message(Msg::App, Msg::Sparse);

	likhdcmp = attmap["LIK_NULL"] * gradient + intercept;
	errstr << "LIKHDCMP   = " << likhdcmp << Sas::message(Msg::App, Msg::Sparse);

	double likhood = attmap["LIK_HOOD"];
	
	bool poscorok;
	poscorok =  likhood > likhdcmp;
	errstr << "RESULT    =  " <<  poscorok << Sas::message(Msg::App, Msg::Sparse);
	return poscorok;
}

void readkeywords( Table *table , Attmap& attmap ) 
{

	// keywords to be read
	// istrstream keywords_to_read("RAOFFSET RAOFFERR DEOFFSET DEOFFERR ROT_CORR ROT_ERR NMATCHES LIK_HOOD");

	string att;
	istringstream keywords_to_read("RAOFFSET DEOFFSET LIK_HOOD LIK_NULL");
	while( keywords_to_read >> att )
	{ 
		if (table->hasAttribute(att)) 
		{
			attmap[att] = table->attribute(att)->asReal();
		} 
		else
		{
			errstr << "Attribute missing: "  << att << Sas::warning("AttributeMissing");		
		}
	}
}

void writeattribute( Table* table , const string& name , const string& value , const string& comment = "" )
{
	(table->hasAttribute(name))
		? *table->attribute(name) = value 
		: *table->addAttribute(name , comment) = value;
	;
}


void writeattribute( Table* table , const string& name , const double value , const string& comment = "" )
{
	(table->hasAttribute(name))
		? *table->attribute(name) = value 
		: *table->addAttribute(name , comment) = value;
	;
}

void writeattribute( Table* table , const string& name , const bool value , const string& comment = "" )
{
	(table->hasAttribute(name))
		? *table->attribute(name) = value 
		: *table->addAttribute(name , comment) = value;
	;
}

void writeposcorok( Table* table , const bool value)
{
	// POSCOROK
	writeattribute( table , "POSCOROK" , value , "eposcorr positions OK?" );
}

void writesyserrcc( Table* table , const double value )
{
	// SYSERRCC
	writeattribute( table , "SYSERRCC" , value , "[arcsec] systematic src posn error for cat corrln" );
}

void writelikhdcmp( Table* table , const double value )
{
	// LIKHDCMP
	writeattribute( table , "LIKHDCMP" , value , "Munched LIK_NULL value" );
}

void copycols(Table *table) 
{
	// Columns to be copied
	// RA -> RA_CORR
	// DEC -> DEC_CORR
	// LII -> LII_CORR
	// BII -> BII_CORR

	int cols = 4;
	string from[4] = {"RA","DEC","LII","BII"};
	string to[4] = {"RA_CORR","DEC_CORR","LII_CORR","BII_CORR"};

	for(int i = 0; i< cols ; ++i) 
	{
		if (table->hasColumn(to[i]) && table->hasColumn(from[i])) 
		{
			int pos = table->columnNumber(to[i]);
			table->deleteColumn(to[i]);
			table->add(table->column(from[i]) , to[i] , pos);
		}
		else
		{
			errstr << "Unable to overwrite " << to[i] << " with "<< from[i] << ".  One of them does not exists." << Sas::warning("ColumnMissing");
		}

	}
}

