// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
 evalcorr

   Authors:  Duncan John Fyfe - LUX
         
   Description: Evaluate the output of eposcorr
 
   Comments: 
 
  $Id: module.cc,v 1.5 2000/10/02 08:30:13 gvacanti Exp $
*/

#include <string>

#include "param.h"
#include "errstr.h"
#include "Dal.h"
#include "stringutil.h"

#include "Evalcorr.h"
#include <Evaluator.h>

#include <Tracer.h>
#include <errstr.h>

#include "evalcorr.h"

void evalcorr ()
{
PROC
	// Sort out input and output names	
	const string exprtrue("TRUE");	
	const string exprfalse("FALSE");	

	string input = stringParameter("input");
	string output;
	string tablename = "SRCLIST";
	string mode = stringParameter("mode");

	strlist v( breakup(input) );
	if ( v.size() > 1 )
	{
		input = v[0];
		tablename = v[1];
	}

	const bool setsyserrcc = booleanParameter("setsyserrcc");
	const bool setlikhdcmp = booleanParameter("setlikhdcmp");
	const bool recordresult = booleanParameter("recordresult");

	const double corrsyserrcc = realParameter("corrsyserrcc");
	const double uncorrsyserrcc = realParameter("uncorrsyserrcc");
	const double radiusthreshold = realParameter("radiusthreshold");
	const double likhoodthreshold = realParameter("likhoodthreshold");
	const double gradient = realParameter("gradient");
	const double intercept = realParameter("intercept");

	DataSet *set;

	if (mode == "MODIFY")
	{
		set = dataSetServer->open( input , Dal::Modify );
	}
	else 
	{
		output = stringParameter("output");
		if ( ! output.size() )
		{
			errstr << "Output dataset mandatory in mode CLONE"  << Sas::error("OutputDatasetMissing");
		}
		set = dataSetServer->clone(input, output, Dal::Modify);
	}
	
	Table *table;
	table = set->table(tablename);

	Attmap attmap;
	readkeywords(table,attmap);

	/* Accounting to make sure we actually run at least one test */
	bool account = 0;

	// The result we want
	bool poscorok=0;

	// Value generated by XMM2 which we may need to write to an attribute
	double likhdcmp = 0;

	string tests[] = { "1XMM" , "2XMM" , "" };
	string expr = stringParameter("expression");
	const string exprcpy(expr);
	Testmap testmap;
	Testmap::const_iterator testpos;

	for( std::string* test = tests ; test->size() ; test++ )
	{
		std::string::size_type exprpos = expr.find( *test , 0);
		while ( exprpos != std::string::npos )
		{
			testpos = testmap.find(*test);
			if ( testpos == testmap.end() )
			{
				if (*test == "1XMM")
				{
					bool thistest = test1xmm( attmap , radiusthreshold , likhoodthreshold );
					testpos = testmap.insert( testmap.begin() , make_pair( *test , thistest ) );
					if (recordresult) writeattribute( table , "TEST1XMM" , thistest , "Result of evalcorr 1XMM test." );
					account = true;
				}
				else if (*test == "2XMM")
				{
					
					bool thistest = test2xmm( attmap , gradient , intercept , likhdcmp );
					testpos = testmap.insert( testmap.begin() , make_pair( *test , thistest ) );
					if (recordresult) writeattribute( table , "TEST2XMM" , thistest , "Result of evalcorr 2XMM test." );
					if (setlikhdcmp) writelikhdcmp(table , likhdcmp);
					account = true;
				}
				else
				{
					// Scintillate scintillate globe magnific,
					// How I ponder your nature specific,
					// Loftily poised in the ether capacious,
					// Closely resembling a gem carbonaceous.
				}
			}

			while ( exprpos != std::string::npos )
			{
				
				expr.replace( exprpos , test->size() , ( testpos->second ? exprtrue : exprfalse )  );
				exprpos = expr.find( *test , exprpos );
			}
		}
	}

	if( !( expr.size() && account ) ) 
	{
		errstr << "Null test. Did nothing"  << Sas::warning("NullTest");
	}
	else if ( ! Evaluator::isValidExpression(expr) )
	{
		errstr << expr << " is not a valid expression." << Sas::warning("InvalidTest");
	}
	else
	{
		errstr << "Expression : " <<  exprcpy << Sas::message(Msg::App, Msg::Sparse);
		errstr << "Evaluate   : " <<  expr << Sas::message(Msg::App, Msg::Sparse);

		Evaluator eval(expr);
		if (eval.isLogic())
		{
			errstr << "evalLogic" <<  Sas::message(Msg::App, Msg::Sparse);
			poscorok = eval.evalLogic() ? 1 : 0;	
		}
		else
		{
			errstr << "evalArith" <<  Sas::message(Msg::App, Msg::Sparse);
			poscorok = eval.evalArith() ? 1 : 0;
		}
		errstr << "POSCOROKR = " <<  poscorok << Sas::message(Msg::App, Msg::Sparse);

		writeposcorok(table , poscorok);

		if (setsyserrcc)
		{
			const double syserrcc = (poscorok) ? corrsyserrcc : uncorrsyserrcc ;
			writesyserrcc(table , syserrcc);
		}

		// position correction was not good enough.  Undo it by copying col -> col_corr.
		if (!poscorok)
		{
			copycols(table);
		}
	}
	dataSetServer->close(set);
ENDPROC
} // evalcorr()
